<?php

namespace Pinpoint\SnowdogMenuGraphQl\Model\Resolver\DataProvider;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\NoSuchEntityException;
use Psr\Log\LoggerInterface;
use Snowdog\Menu\Model\Menu as MenuModel;
use Snowdog\Menu\Model\Menu\NodeRepository;
use Snowdog\Menu\Model\MenuRepository;

/**
 * Menu Data Provider.
 */
class Menu
{
    /**
     * @var SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * @var MenuRepository
     */
    protected $menuRepository;

    /**
     * @var NodeRepository
     */
    protected $nodeRepository;

    /**
     * @var CategoryRepositoryInterface
     */
    protected $categoryRepository;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param MenuRepository $menuRepository
     * @param NodeRepository $nodeRepository
     * @param CategoryRepositoryInterface $categoryRepository
     * @param LoggerInterface $logger
     */
    public function __construct(
        SearchCriteriaBuilder $searchCriteriaBuilder,
        MenuRepository $menuRepository,
        NodeRepository $nodeRepository,
        CategoryRepositoryInterface $categoryRepository,
        LoggerInterface $logger
    ) {
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->menuRepository = $menuRepository;
        $this->nodeRepository = $nodeRepository;
        $this->categoryRepository = $categoryRepository;
        $this->logger = $logger;
    }

    /**
     * @param int $menuId
     * @return MenuModel
     * @throws NoSuchEntityException
     */
    public function getMenu(int $menuId): MenuModel
    {
        return $this->menuRepository->getById($menuId);
    }

    /**
     * @param int $menuId
     * @return array
     */
    public function getNodes(int $menuId): array
    {
        $data = [];
        $nodes = $this->nodeRepository->getByMenu($menuId);
        foreach ($nodes as $node) {
            $data[] = [
                'name' => $node['title'],
                'cssClass' => $node['classes'],
                'nodeType' => $node['type'],
                'content' => $this->getNodeContent($node),
                'position' => $node['position'],
                'id' => $node['node_id'],
                'parent' => $node['parent_id'] ?? 0,
                'level' => $node['level'],
            ];
        }
        return $data;
    }

    /**
     * @param $node
     * @return mixed|null
     */
    protected function getNodeContent($node)
    {
        $content = $node['content'];
        if ($node['type'] === 'category') {
            try {
                $category = $this->categoryRepository->get($node['content']);
                $content = $category->getUrlKey();
            } catch (NoSuchEntityException $exception) {
                $this->logger->critical($exception->getMessage());
                $content = null;
            }
        }
        return $content;
    }
}
