<?php

namespace Pinpoint\SnowdogMenuGraphQl\Model\Resolver;

use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Exception\GraphQlNoSuchEntityException;
use Magento\Framework\GraphQl\Query\Resolver\ContextInterface;
use Magento\Framework\GraphQl\Query\Resolver\Value;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;
use Snowdog\Menu\Model\Menu as MenuModel;
use Pinpoint\SnowdogMenuGraphQl\Model\Resolver\DataProvider\Menu as MenuDataProvider;

/**
 * Add menu resolver.
 */
class Menu implements ResolverInterface
{
    /**
     * @var MenuDataProvider
     */
    protected $menuDataProvider;

    /**
     * Menu constructor.
     * @param MenuDataProvider $menuDataProvider
     */
    public function __construct(MenuDataProvider $menuDataProvider)
    {
        $this->menuDataProvider = $menuDataProvider;
    }

    /**
     * Fetches the data from persistence models and format it according to the GraphQL schema.
     *
     * @param \Magento\Framework\GraphQl\Config\Element\Field $field
     * @param ContextInterface $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @throws \Exception
     * @return mixed|Value
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        if (!isset($args[MenuModel::MENU_ID])) {
            throw new GraphQlInputException(__('Menu ID must be provided.'));
        }
        try {
            $menu = $this->menuDataProvider->getMenu($args[MenuModel::MENU_ID]);
            $nodes = $this->menuDataProvider->getNodes($args[MenuModel::MENU_ID]);
            return [
                'title' => $menu->getData('title'),
                'id' => $menu->getData('identifier'),
                'cssClass' => $menu->getData('css_class'),
                'nodes' => $nodes
            ];
        } catch (NoSuchEntityException $exception) {
            throw new GraphQlNoSuchEntityException(__($exception), $exception);
        }
    }
}
